@@ -47,7 +47,10 @@ def test_keepalive_two_requests_success(loop, test_client):
 
     app = web.Application(loop=loop)
     app.router.add_route('GET', '/', handler)
-    client = yield from test_client(app)
+
+    connector = aiohttp.TCPConnector(loop=loop, limit=1)
+    client = yield from test_client(app, connector=connector)
+
     resp1 = yield from client.get('/')
     yield from resp1.read()
     resp2 = yield from client.get('/')
@@ -66,7 +69,9 @@ def test_keepalive_response_released(loop, test_client):
 
     app = web.Application(loop=loop)
     app.router.add_route('GET', '/', handler)
-    client = yield from test_client(app)
+
+    connector = aiohttp.TCPConnector(loop=loop, limit=1)
+    client = yield from test_client(app, connector=connector)
 
     resp1 = yield from client.get('/')
     yield from resp1.release()
@@ -88,7 +93,9 @@ def test_keepalive_server_force_close_connection(loop, test_client):
 
     app = web.Application(loop=loop)
     app.router.add_route('GET', '/', handler)
-    client = yield from test_client(app)
+
+    connector = aiohttp.TCPConnector(loop=loop, limit=1)
+    client = yield from test_client(app, connector=connector)
 
     resp1 = yield from client.get('/')
     resp1.close()
@@ -337,7 +344,7 @@ def test_format_task_get(test_server, loop):
 def test_str_params(loop, test_client):
     @asyncio.coroutine
     def handler(request):
-        assert 'q=t+est' in request.query_string
+        assert 'q=t est' in request.query_string
         return web.Response()
 
     app = web.Application(loop=loop)
@@ -440,7 +447,9 @@ def test_keepalive_closed_by_server(loop, test_client):
 
     app = web.Application(loop=loop)
     app.router.add_route('GET', '/', handler)
-    client = yield from test_client(app)
+
+    connector = aiohttp.TCPConnector(loop=loop, limit=1)
+    client = yield from test_client(app, connector=connector)
 
     resp1 = yield from client.get('/')
     val1 = yield from resp1.read()
@@ -541,6 +550,46 @@ def test_timeout_on_reading_headers(loop, test_client):
         yield from client.get('/', timeout=0.01)
 
 
+@asyncio.coroutine
+def test_timeout_on_conn_reading_headers(loop, test_client):
+    # tests case where user did not set a connection timeout
+
+    @asyncio.coroutine
+    def handler(request):
+        resp = web.StreamResponse()
+        yield from asyncio.sleep(0.1, loop=loop)
+        yield from resp.prepare(request)
+        return resp
+
+    app = web.Application(loop=loop)
+    app.router.add_route('GET', '/', handler)
+
+    conn = aiohttp.TCPConnector(loop=loop)
+    client = yield from test_client(app, connector=conn)
+
+    with pytest.raises(asyncio.TimeoutError):
+        yield from client.get('/', timeout=0.01)
+
+
+@asyncio.coroutine
+def test_timeout_on_session_read_timeout(loop, test_client):
+    @asyncio.coroutine
+    def handler(request):
+        resp = web.StreamResponse()
+        yield from asyncio.sleep(0.1, loop=loop)
+        yield from resp.prepare(request)
+        return resp
+
+    app = web.Application(loop=loop)
+    app.router.add_route('GET', '/', handler)
+
+    conn = aiohttp.TCPConnector(loop=loop)
+    client = yield from test_client(app, connector=conn, read_timeout=0.01)
+
+    with pytest.raises(asyncio.TimeoutError):
+        yield from client.get('/', timeout=None)
+
+
 @asyncio.coroutine
 def test_timeout_on_reading_data(loop, test_client):
 
