@@ -84,14 +84,21 @@ class TestParseHeaders(unittest.TestCase):
             parser = protocol.HttpParser(8190, 32768, 5)
             parser.parse_headers(
                 [b'', b'test: line data data\r\n', b'data\r\n', b'\r\n'])
-        self.assertIn("limit request headers fields size", str(cm.exception))
+        self.assertIn("request header field TEST", str(cm.exception))
 
     def test_max_continuation_headers_size(self):
         with self.assertRaises(errors.LineTooLong) as cm:
             parser = protocol.HttpParser(8190, 32768, 5)
             parser.parse_headers([b'', b'test: line\r\n',
                                   b' test\r\n', b'\r\n'])
-        self.assertIn("limit request headers fields size", str(cm.exception))
+        self.assertIn("request header field TEST", str(cm.exception))
+
+    def test_max_header_size(self):
+        with self.assertRaises(errors.LineTooLong) as cm:
+            parser = protocol.HttpParser(5, 5, 5)
+            parser.parse_headers(
+                [b'', b'test: line data data\r\n', b'data\r\n', b'\r\n'])
+        self.assertIn("request header", str(cm.exception))
 
     def test_invalid_header(self):
         with self.assertRaisesRegex(
@@ -154,6 +161,13 @@ class TestDeflateBuffer(unittest.TestCase):
 
         self.assertRaises(errors.ContentEncodingError, dbuf.feed_eof)
 
+    def test_empty_body(self):
+        buf = aiohttp.FlowControlDataQueue(self.stream)
+        dbuf = protocol.DeflateBuffer(buf, 'deflate')
+        dbuf.feed_eof()
+
+        self.assertTrue(buf.at_eof())
+
 
 class TestParsePayload(unittest.TestCase):
 
