@@ -3,6 +3,7 @@ import pytest
 import aiohttp
 from aiohttp import helpers, web
 
+
 async def test_client_ws_async_for(loop, test_client):
     items = ['q1', 'q2', 'q3']
 
@@ -53,6 +54,31 @@ async def test_client_ws_async_with(loop, test_server):
         assert ws.closed
 
 
+async def test_client_ws_async_with_send(loop, test_server):
+    # send_xxx methods have to return awaitable objects
+
+    async def handler(request):
+        ws = web.WebSocketResponse()
+        await ws.prepare(request)
+        msg = await ws.receive()
+        ws.send_str(msg.data + '/answer')
+        await ws.close()
+        return ws
+
+    app = web.Application(loop=loop)
+    app.router.add_route('GET', '/', handler)
+
+    server = await test_server(app)
+
+    async with aiohttp.ClientSession(loop=loop) as client:
+        async with client.ws_connect(server.make_url('/')) as ws:
+            await ws.send_str('request')
+            msg = await ws.receive()
+            assert msg.data == 'request/answer'
+
+        assert ws.closed
+
+
 async def test_client_ws_async_with_shortcut(loop, test_server):
 
     async def handler(request):
